/*
 * Simple fragment shader for calculating output fragment color.
 */

#version 130          // delete this line if using GLSL 1.2
precision mediump float;  // delete this line if using GLSL 1.2

// uniform shader-parameters
uniform vec4 cam_position;
uniform vec3 light_position;

// light components
uniform vec4 diffuse_color;
uniform vec4 specular_color;
uniform vec4 ambient_color;

// material constants for light components
uniform vec4 diffuse_mat;
uniform vec4 specular_mat;
uniform vec4 ambient_mat;

// the exponent for the specular component
uniform float spec_power;

// sampler holding texture data
uniform sampler2D colorTexture;

// fragment-shader input variables
in vec4 frag_color;
in vec3 world_normal;
in vec4 world_position;
in vec2 fragTexCoord;

// fragment-shader output variable (-> stored in the frame-buffer, i.e. "the pixel you see")
out vec4 fragColor;


void main()
{
	// renormalize and homogenize input variables
	vec3 normal = normalize(world_normal);
    vec3 position = world_position.xyz / world_position.w;
    
    // calculate the light-direction
    vec3 light_dir = normalize(light_position - position);
	
	 // calculate haveway vector
	vec3 view = normalize(cam_position.xyz - position);
	vec3 halfWay = normalize(light_dir + view);
	
	//get texel color
	vec4 texel = texture(colorTexture, fragTexCoord);
	
	// calculate lighting
    vec4 ambient = ambient_color* ambient_mat * texel;
    vec4 diffuse = texel * diffuse_color * diffuse_mat * max(0.0, dot(normal, light_dir));
    
    //calculate specular
    float spec = max(0.0, dot(normal, halfWay));
    float factor = pow(spec, spec_power);
    vec4 specular = specular_color * specular_mat * factor;

    
    // write color to output
    fragColor = ambient + diffuse + specular;
    //fragColor = vec4(0.0, 1.0, 0.0, 1.0);
}
